@@ -559,6 +559,7 @@ SEXP Rf_asChar(SEXP);
 SEXP Rf_coerceVector(SEXP, SEXPTYPE);
 SEXP Rf_PairToVectorList(SEXP x);
 SEXP Rf_VectorToPairList(SEXP x);
+SEXP Rf_asCharacterFactor(SEXP x);
 int Rf_asLogical(SEXP x);
 int Rf_asInteger(SEXP x);
 double Rf_asReal(SEXP x);
@@ -653,7 +654,8 @@ void R_Reprotect(SEXP, PROTECT_INDEX);
 SEXP R_tryEval(SEXP, SEXP, int *);
 
 Rboolean Rf_isS4(SEXP);
-SEXP Rf_asS4(SEXP, Rboolean);
+SEXP Rf_asS4(SEXP, Rboolean, int);
+SEXP Rf_S3Class(SEXP);
 
 typedef enum {
     CE_NATIVE = 0,
@@ -853,6 +855,7 @@ int R_system(const char *);
 #define applyClosure		Rf_applyClosure
 #define arraySubscript		Rf_arraySubscript
 #define asChar			Rf_asChar
+#define asCharacterFactor	Rf_asCharacterFactor
 #define asComplex		Rf_asComplex
 #define asInteger		Rf_asInteger
 #define asLogical		Rf_asLogical
@@ -964,6 +967,7 @@ int R_system(const char *);
 #define protect			Rf_protect
 #define reEnc			Rf_reEnc
 #define rownamesgets		Rf_rownamesgets
+#define S3Class                 Rf_S3Class
 #define ScalarComplex		Rf_ScalarComplex
 #define ScalarInteger		Rf_ScalarInteger
 #define ScalarLogical		Rf_ScalarLogical
