@@ -152,7 +152,8 @@ SEXP attribute_hidden do_matrix(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    ;
 	}
     }
-    if(!isNull(dimnames)) ans = dimnamesgets(ans, dimnames);
+    if(!isNull(dimnames)&& length(dimnames) > 0)
+	ans = dimnamesgets(ans, dimnames);
     UNPROTECT(1);
     return ans;
 }
@@ -290,6 +291,15 @@ SEXP DropDims(SEXP x)
 	setAttrib(x, R_DimNamesSymbol, R_NilValue);
 	setAttrib(x, R_DimSymbol, R_NilValue);
 	setAttrib(x, R_NamesSymbol, newnames);
+	/* FIXME: the following is desirable, but pointless as long as
+	   subset.c & others have a contrary version that leaves the
+	   S4 class in, incorrectly, in the case of vectors.  JMC
+	   3/3/09 */
+/* 	if(IS_S4_OBJECT(x)) {/\* no longer valid subclass of array or
+ 	matrix *\/ */
+/* 	    setAttrib(x, R_ClassSymbol, R_NilValue); */
+/* 	    UNSET_S4_OBJECT(x); */
+/* 	} */
 	UNPROTECT(1);
     } else {
 	/* We have a lower dimensional array. */
@@ -592,7 +602,7 @@ SEXP attribute_hidden do_matprod(SEXP call, SEXP op, SEXP args, SEXP rho)
     sym = isNull(y);
     if (sym && (PRIMVAL(op) > 0)) y = x;
     if ( !(isNumeric(x) || isComplex(x)) || !(isNumeric(y) || isComplex(y)) )
-	errorcall(call, _("requires numeric matrix/vector arguments"));
+	errorcall(call, _("requires numeric/complex matrix/vector arguments"));
 
     xdims = getAttrib(x, R_DimSymbol);
     ydims = getAttrib(y, R_DimSymbol);
@@ -619,7 +629,7 @@ SEXP attribute_hidden do_matprod(SEXP call, SEXP op, SEXP args, SEXP rho)
 	if (PRIMVAL(op) == 0) {
 	    if (LENGTH(x) == nry) {	/* x as row vector */
 		nrx = 1;
-		ncx = LENGTH(x);
+		ncx = nry; /* == LENGTH(x) */
 	    }
 	    else if (nry == 1) {	/* x as col vector */
 		nrx = LENGTH(x);
@@ -628,7 +638,7 @@ SEXP attribute_hidden do_matprod(SEXP call, SEXP op, SEXP args, SEXP rho)
 	}
 	else { /* crossprod */
 	    if (LENGTH(x) == nry) {	/* x is a col vector */
-		nrx = LENGTH(x);
+		nrx = nry; /* == LENGTH(x) */
 		ncx = 1;
 	    }
 	}
@@ -640,7 +650,7 @@ SEXP attribute_hidden do_matprod(SEXP call, SEXP op, SEXP args, SEXP rho)
 	ncy = 0;
 	if (PRIMVAL(op) == 0) {
 	    if (LENGTH(y) == ncx) {	/* y as col vector */
-		nry = LENGTH(y);
+		nry = ncx; /* == LENGTH(y) */
 		ncy = 1;
 	    }
 	    else if (ncx == 1) {	/* y as row vector */
@@ -650,7 +660,7 @@ SEXP attribute_hidden do_matprod(SEXP call, SEXP op, SEXP args, SEXP rho)
 	}
 	else {
 	    if (LENGTH(y) == nrx) {	/* y is a col vector */
-		nry = LENGTH(y);
+		nry = nrx; /* == LENGTH(y) */
 		ncy = 1;
 	    }
 	}
