@@ -19,22 +19,6 @@
  *  http://www.r-project.org/Licenses/
  */
 
-/* <UTF8>
-   abbreviate needs to be fixed, if possible.
-
-   Changes already made:
-   abbreviate needs to be fixed, if possible, but warns for now.
-   Regex code should be OK, substitution does ASCII comparisons only.
-   charToRaw/rawToChar should work at byte level, so is OK.
-   agrep needed to work at char level.
-   make.names worked at byte not char level.
-   substr() should work at char not byte level.
-   Semantics of nchar() have been fixed.
-   regexpr returned pos and match length in bytes not chars.
-   tolower/toupper added wchar versions
-   chartr works at char not byte level.
- */
-
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
@@ -90,11 +74,11 @@ SEXP attribute_hidden do_nzchar(SEXP call, SEXP op, SEXP args, SEXP env)
     checkArity(op, args);
     PROTECT(x = coerceVector(CAR(args), STRSXP));
     if (!isString(x))
-	error(_("nchar() requires a character vector"));
+	error(_("'%s' requires a character vector"), "nzchar()");
     len = LENGTH(x);
     PROTECT(ans = allocVector(LGLSXP, len));
     for (i = 0; i < len; i++)
-	LOGICAL(ans)[i] = length(STRING_ELT(x, i)) > 0;
+	LOGICAL(ans)[i] = LENGTH(STRING_ELT(x, i)) > 0;
     UNPROTECT(2);
     return ans;
 }
@@ -114,7 +98,7 @@ SEXP attribute_hidden do_nchar(SEXP call, SEXP op, SEXP args, SEXP env)
     checkArity(op, args);
     PROTECT(x = coerceVector(CAR(args), STRSXP));
     if (!isString(x))
-	error(_("nchar() requires a character vector"));
+	error(_("'%s' requires a character vector"), "nchar()");
     len = LENGTH(x);
     stype = CADR(args);
     if (!isString(stype) || LENGTH(stype) != 1)
@@ -134,20 +118,17 @@ SEXP attribute_hidden do_nchar(SEXP call, SEXP op, SEXP args, SEXP env)
 	}
 	if (strncmp(type, "bytes", ntype) == 0) {
 	    /* This case counts embedded nuls */
-	    INTEGER(s)[i] = length(sxi);
+	    INTEGER(s)[i] = LENGTH(sxi);
 	} else if (strncmp(type, "chars", ntype) == 0) {
 #ifdef SUPPORT_MBCS
 	    /* only on Windows will non-representable UTF-8 chars be
-	       usefully ouptut as chars and not <U+xxxx> */
-# ifdef Win32
+	       usefully output as chars and not <U+xxxx> */
 	    if (IS_UTF8(sxi)) { /* assume this is valid */
 		const char *p = CHAR(sxi);
 		nc = 0;
 		for( ; *p; p +=utf8clen(*p)) nc++;
 		INTEGER(s)[i] = nc;
-	    } else
-# endif
-	    if (mbcslocale) {
+	    } else if (mbcslocale) {
 		nc = mbstowcs(NULL, translateChar(sxi), 0);
 		if (!allowNA && nc < 0)
 		    error(_("invalid multibyte string %d"), i+1);
@@ -157,7 +138,6 @@ SEXP attribute_hidden do_nchar(SEXP call, SEXP op, SEXP args, SEXP env)
 		INTEGER(s)[i] = strlen(translateChar(sxi));
 	} else if (strncmp(type, "width", ntype) == 0) {
 #ifdef SUPPORT_MBCS
-# ifdef Win32
 	    if (IS_UTF8(sxi)) { /* assume this is valid */
 		const char *p = CHAR(sxi);
 		wchar_t wc1;
@@ -167,9 +147,7 @@ SEXP attribute_hidden do_nchar(SEXP call, SEXP op, SEXP args, SEXP env)
 		    nc +=Ri18n_wcwidth(wc1);
 		}
 		INTEGER(s)[i] = nc;
-	    } else
-# endif
-	    if (mbcslocale) {
+	    } else if (mbcslocale) {
 		xi = translateChar(sxi);
 		nc = mbstowcs(NULL, xi, 0);
 		if (nc >= 0) {
@@ -286,8 +264,7 @@ SEXP attribute_hidden do_substr(SEXP call, SEXP op, SEXP args, SEXP env)
 static void
 substrset(char *buf, const char *const str, cetype_t ienc, int sa, int so)
 {
-    /* Replace the substring buf[sa:so] by str[],
-       or as much as str provides */
+    /* Replace the substring buf[sa:so] by str[] */
     int i, in = 0, out = 0;
 
     if (ienc == CE_UTF8) {
@@ -1038,7 +1015,7 @@ SEXP attribute_hidden do_grep(SEXP call, SEXP op, SEXP args, SEXP env)
     SEXP pat, vec, ind, ans;
     regex_t reg;
     int i, j, n, nmatches = 0, cflags = 0, ov, erroffset, ienc, rc;
-    int igcase_opt, extended_opt, value_opt, perl_opt, fixed_opt, useBytes;
+    int igcase_opt, extended_opt, value_opt, perl_opt, fixed_opt, useBytes, invert;
     const char *cpat, *errorptr;
     pcre *re_pcre = NULL /* -Wall */;
     pcre_extra *re_pe = NULL;
@@ -1054,12 +1031,14 @@ SEXP attribute_hidden do_grep(SEXP call, SEXP op, SEXP args, SEXP env)
     perl_opt = asLogical(CAR(args)); args = CDR(args);
     fixed_opt = asLogical(CAR(args)); args = CDR(args);
     useBytes = asLogical(CAR(args)); args = CDR(args);
+    invert = asLogical(CAR(args));
     if (igcase_opt == NA_INTEGER) igcase_opt = 0;
     if (extended_opt == NA_INTEGER) extended_opt = 1;
     if (value_opt == NA_INTEGER) value_opt = 0;
     if (perl_opt == NA_INTEGER) perl_opt = 0;
     if (fixed_opt == NA_INTEGER) fixed_opt = 0;
     if (useBytes == NA_INTEGER) useBytes = 0;
+    if (invert == NA_INTEGER) invert = 0;
     if (fixed_opt && igcase_opt)
 	warning(_("argument '%s' will be ignored"), "ignore.case = TRUE");
     if (fixed_opt && perl_opt)
@@ -1169,7 +1148,7 @@ SEXP attribute_hidden do_grep(SEXP call, SEXP op, SEXP args, SEXP env)
 		    INTEGER(ind)[i] = 1;
 	    } else if (regexec(&reg, s, 0, NULL, 0) == 0) LOGICAL(ind)[i] = 1;
 	}
-	if (LOGICAL(ind)[i]) nmatches++;
+	if (invert ^ LOGICAL(ind)[i]) nmatches++;
     }
     if (fixed_opt);
     else if (perl_opt) {
@@ -1179,17 +1158,22 @@ SEXP attribute_hidden do_grep(SEXP call, SEXP op, SEXP args, SEXP env)
     } else
 	regfree(&reg);
 
+    if (PRIMVAL(op)) {/* grepl case */
+	UNPROTECT(1);
+	return ind;
+    }	
+
     if (value_opt) {
 	SEXP nmold = getAttrib(vec, R_NamesSymbol), nm;
 	ans = allocVector(STRSXP, nmatches);
 	for (i = 0, j = 0; i < n ; i++)
-	    if (LOGICAL(ind)[i])
+	    if (invert ^ LOGICAL(ind)[i])
 		SET_STRING_ELT(ans, j++, STRING_ELT(vec, i));
 	/* copy across names and subset */
 	if (!isNull(nmold)) {
 	    nm = allocVector(STRSXP, nmatches);
 	    for (i = 0, j = 0; i < n ; i++)
-		if (LOGICAL(ind)[i])
+		if (invert ^ LOGICAL(ind)[i])
 		    SET_STRING_ELT(nm, j++, STRING_ELT(nmold, i));
 	    setAttrib(ans, R_NamesSymbol, nm);
 	}
@@ -1197,7 +1181,7 @@ SEXP attribute_hidden do_grep(SEXP call, SEXP op, SEXP args, SEXP env)
 	ans = allocVector(INTSXP, nmatches);
 	j = 0;
 	for (i = 0 ; i < n ; i++)
-	    if (LOGICAL(ind)[i]) INTEGER(ans)[j++] = i + 1;
+	    if (invert ^ LOGICAL(ind)[i]) INTEGER(ans)[j++] = i + 1;
     }
     UNPROTECT(1);
     return ans;
@@ -2809,8 +2793,9 @@ SEXP attribute_hidden do_rawToChar(SEXP call, SEXP op, SEXP args, SEXP env)
 	len = LENGTH(x);
 	PROTECT(ans = allocVector(STRSXP, 1));
 	/* String is not necessarily 0-terminated and may contain nuls
-	   so don't use mkChar */
-	SET_STRING_ELT(ans, 0, mkCharLen((const char *)RAW(x), len));
+	   so don't use mkString */
+	SET_STRING_ELT(ans, 0,
+		       mkCharLenCE((const char *)RAW(x), len, CE_NATIVE));
     }
     UNPROTECT(1);
     return ans;
@@ -2857,19 +2842,20 @@ SEXP attribute_hidden do_rawToBits(SEXP call, SEXP op, SEXP args, SEXP env)
 
 SEXP attribute_hidden do_intToBits(SEXP call, SEXP op, SEXP args, SEXP env)
 {
-    SEXP ans, x = CAR(args);
+    SEXP ans, x;
     int i, j = 0, k;
     unsigned int tmp;
-
+    
+    PROTECT(x = coerceVector(CAR(args), INTSXP));
     if (!isInteger(x))
-	error(_("argument 'x' must be a integer vector"));
+	error(_("argument 'x' must be an integer vector"));
     PROTECT(ans = allocVector(RAWSXP, 32*LENGTH(x)));
     for (i = 0; i < LENGTH(x); i++) {
 	tmp = (unsigned int) INTEGER(x)[i];
 	for (k = 0; k < 32; k++, tmp >>= 1)
 	    RAW(ans)[j++] = tmp & 0x1;
     }
-    UNPROTECT(1);
+    UNPROTECT(2);
     return ans;
 }
 
@@ -3049,13 +3035,15 @@ static size_t inttomb(char *s, const int wc)
 
 SEXP attribute_hidden do_intToUtf8(SEXP call, SEXP op, SEXP args, SEXP env)
 {
-    SEXP ans, c, x = CAR(args);
-    int i, nc = LENGTH(x), multiple, len, used;
-    char buf[10];
+    SEXP ans, x;
+    int i, nc, multiple, len, used;
+    char buf[10], *tmp;
 
     checkArity(op, args);
+    PROTECT(x = coerceVector(CAR(args), INTSXP));
     if (!isInteger(x))
 	error(_("argument 'x' must be an integer vector"));
+    nc = LENGTH(x);
     multiple = asLogical(CADR(args));
     if (multiple == NA_LOGICAL)
 	error(_("argument 'multiple' must be TRUE or FALSE"));
@@ -3068,21 +3056,20 @@ SEXP attribute_hidden do_intToUtf8(SEXP call, SEXP op, SEXP args, SEXP env)
 	}
 	/* do we want to copy e.g. names here? */
     } else {
+	/* Note that this gives zero length for input '0', so it is omitted */
 	for (i = 0, len = 0; i < nc; i++)
 	    len += inttomb(NULL, INTEGER(x)[i]);
-	PROTECT(ans = allocVector(STRSXP, 1));
-	/* String is not necessarily 0-terminated and may contain nuls
-	   so don't use mkChar */
-	c = allocString(len); /* adds zero terminator */
+	tmp = alloca(len+1); tmp[len] = '\0';
+	R_CheckStack();
 	for (i = 0, len = 0; i < nc; i++) {
 	    used = inttomb(buf, INTEGER(x)[i]);
-	    strncpy(CHAR_RW(c) + len, buf, used);
-	    SET_UTF8(c);
+	    strncpy(tmp + len, buf, used);
 	    len += used;
 	}
-	SET_STRING_ELT(ans, 0, c);
+	PROTECT(ans = allocVector(STRSXP, 1));
+	SET_STRING_ELT(ans, 0, mkCharLenCE(tmp, len, CE_UTF8));
     }
-    UNPROTECT(1);
+    UNPROTECT(2);
     return ans;
 }
 
