@@ -1489,6 +1489,7 @@ static void vsignalWarning(SEXP call, const char *format, va_list ap)
     char buf[BUFSIZE];
     SEXP hooksym, quotesym, hcall, qcall;
 
+    saveJitStackPointer(1);
     hooksym = install(".signalSimpleWarning");
     quotesym = install("quote");
     if (SYMVALUE(hooksym) != R_UnboundValue &&
@@ -1502,6 +1503,7 @@ static void vsignalWarning(SEXP call, const char *format, va_list ap)
 	UNPROTECT(3);
     }
     else vwarningcall_dflt(call, format, ap);
+    saveJitStackPointer(0);
 }
 
 static void gotoExitingHandler(SEXP cond, SEXP call, SEXP entry)
@@ -1800,7 +1802,7 @@ SEXP attribute_hidden do_addTryHandlers(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     checkArity(op, args);
     if (R_GlobalContext == R_ToplevelContext ||
-	! R_GlobalContext->callflag & CTXT_FUNCTION)
+	! (R_GlobalContext->callflag & CTXT_FUNCTION))
 	errorcall(call, _("not in a try context"));
     SET_RESTART_BIT_ON(R_GlobalContext->callflag);
     R_InsertRestartHandlers(R_GlobalContext, FALSE);
