@@ -393,7 +393,7 @@ SEXP EnsureLocal(SEXP symbol, SEXP rho)
 
     vl = eval(symbol, ENCLOS(rho));
     if (vl == R_UnboundValue)
-        error(_("object \"%s\" not found"), CHAR(PRINTNAME(symbol)));
+        error(_("object '%s' not found"), CHAR(PRINTNAME(symbol)));
 
     PROTECT(vl = duplicate(vl));
     defineVar(symbol, vl, rho);
@@ -682,7 +682,7 @@ static SEXP applydefine(SEXP call, SEXP op, SEXP args, SEXP rho)
     UNPROTECT(5);
     endcontext(&cntxt); /* which does not run the remove */
     unbindVar(R_TmpvalSymbol, rho);
-#ifdef CONSERVATIVE_COPYING
+#ifdef CONSERVATIVE_COPYING /* not default */
     return duplicate(saverhs);
 #else
     /* we do not duplicate the value, so to be conservative mark the
@@ -709,14 +709,21 @@ SEXP attribute_hidden do_set(SEXP call, SEXP op, SEXP args, SEXP rho)
     SEXP s;
     if (length(args) != 2)
         WrongArgCount(asym[PRIMVAL(op)]);
-    if (isString(CAR(args)))
-        SETCAR(args, install(translateChar(STRING_ELT(CAR(args), 0))));
+    if (isString(CAR(args))) {
+	/* fix up a duplicate or args and recursively call do_set */
+	SEXP val;
+	PROTECT(args = duplicate(args));
+	SETCAR(args, install(translateChar(STRING_ELT(CAR(args), 0))));
+	val = do_set(call, op, args, rho);
+	UNPROTECT(1);
+	return val;
+    }
 
     switch (PRIMVAL(op)) {
     case 1: case 3:                                     /* <-, = */
         if (isSymbol(CAR(args))) {
             PROTECT(s = eval(CADR(args), rho));
-#ifdef CONSERVATIVE_COPYING
+#ifdef CONSERVATIVE_COPYING /* not default */
             if (NAMED(s))
             {
                 SEXP t;
@@ -807,14 +814,8 @@ SEXP attribute_hidden evalList(SEXP el, SEXP rho, SEXP op)
 	    }
 	    else if (h != R_MissingArg)
 		error(_("'...' used in an incorrect context"));
-/* Uncomment the following to restore old behavior */
-/* #define OLDMISSING */
-#ifdef OLDMISSING
-	} else if (CAR(el) != R_MissingArg) {
-#else
 	} else if (!(CAR(el) == R_MissingArg ||
                  (isSymbol(CAR(el)) && R_isMissing(CAR(el), rho)))) {
-#endif
 	    SETCDR(tail, CONS(eval(CAR(el), rho), R_NilValue));
 	    tail = CDR(tail);
 	    SET_TAG(tail, CreateTag(TAG(el)));
@@ -876,12 +877,8 @@ SEXP attribute_hidden evalListKeepMissing(SEXP el, SEXP rho)
 	    else if(h != R_MissingArg)
 		error(_("'...' used in an incorrect context"));
 	}
-#ifdef OLDMISSING
-	else if (CAR(el) == R_MissingArg) {
-#else
 	else if (CAR(el) == R_MissingArg ||
                  (isSymbol(CAR(el)) && R_isMissing(CAR(el), rho))) {
-#endif
 	    SETCDR(tail, CONS(R_MissingArg, R_NilValue));
 	    tail = CDR(tail);
 	    SET_TAG(tail, CreateTag(TAG(el)));
@@ -1078,6 +1075,25 @@ SEXP attribute_hidden do_eval(SEXP call, SEXP op, SEXP args, SEXP rho)
 }
 
 
+SEXP attribute_hidden do_withVisible(SEXP call, SEXP op, SEXP args, SEXP rho)
+{
+    SEXP x, nm, ret;
+
+    checkArity(op, args);
+    x = CAR(args);
+    x = eval(x, rho);
+    PROTECT(x);
+    PROTECT(ret = allocVector(VECSXP, 2));
+    PROTECT(nm = allocVector(STRSXP, 2));
+    SET_STRING_ELT(nm, 0, mkChar("value"));
+    SET_STRING_ELT(nm, 1, mkChar("visible"));
+    SET_VECTOR_ELT(ret, 0, x);
+    SET_VECTOR_ELT(ret, 1, ScalarLogical(R_Visible));
+    setAttrib(ret, R_NamesSymbol, nm);
+    UNPROTECT(3);
+    return ret;
+}
+
 SEXP attribute_hidden do_recall(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     RCNTXT *cptr;
@@ -1303,13 +1319,39 @@ static void findmethod(SEXP Class, const char *group, const char *generic,
     *which = whichclass;
 }
 
+/* a portion of the logic of R_data_class2, but not producing
+   "matrix", "array" classes or vector types as classes  */
+static SEXP data_class_group(SEXP obj) {
+    SEXP klass = getAttrib(obj, R_ClassSymbol);
+      if(length(klass) > 0) {
+	if(IS_S4_OBJECT(obj) && TYPEOF(obj) != S4SXP) { 
+	    /* try to return an S3Class slot, but NOT matrix/array */
+	    /* The S4 class is included for compatibility with
+	       the deprecated practice of defining S3 methods 
+	       for S4 classes.  Someday this should be disallowed. 
+	       JMC iii.9.09 */
+  	    SEXP s3class = S3Class(obj);
+	    if(s3class != R_NilValue) {
+	        SEXP value; int i, n = length(s3class);
+		PROTECT(value =  allocVector(STRSXP, n+1));
+		SET_STRING_ELT(value, 0, STRING_ELT(klass, 0));
+		for(i=0; i<n; i++)
+		  SET_STRING_ELT(value, i+1, STRING_ELT(s3class, i));
+		UNPROTECT(1);
+		return value;
+	    }
+	}
+      }
+      return(klass);
+}
+
 attribute_hidden
 int DispatchGroup(const char* group, SEXP call, SEXP op, SEXP args, SEXP rho,
                   SEXP *ans)
 {
     int i, j, nargs, lwhich, rwhich, set;
     SEXP lclass, s, t, m, lmeth, lsxp, lgr, newrho;
-    SEXP rclass, rmeth, rgr, rsxp;
+    SEXP rclass, rmeth, rgr, rsxp, value;
     char lbuf[512], rbuf[512], generic[128], *pt;
     Rboolean useS4 = TRUE, isOps = FALSE;
 
@@ -1320,8 +1362,8 @@ int DispatchGroup(const char* group, SEXP call, SEXP op, SEXP args, SEXP rho,
        rejected by the code following the string examination code
        below */
     if (args != R_NilValue && ! isObject(CAR(args)) &&
-        (CDR(args) == R_NilValue || ! isObject(CADR(args))))
-        return 0;
+	(CDR(args) == R_NilValue || ! isObject(CADR(args))))
+	return 0;
 
     isOps = strcmp(group, "Ops") == 0;
 
@@ -1329,90 +1371,120 @@ int DispatchGroup(const char* group, SEXP call, SEXP op, SEXP args, SEXP rho,
     if(length(args) == 1 && !IS_S4_OBJECT(CAR(args))) useS4 = FALSE;
     if(length(args) == 2 &&
        !IS_S4_OBJECT(CAR(args)) && !IS_S4_OBJECT(CADR(args))) useS4 = FALSE;
-    if(useS4 && R_has_methods(op)) {
-        SEXP value;
-        /* Remove argument names to ensure positional matching */
-        if(isOps)
-            for(s = args; s != R_NilValue; s = CDR(s)) SET_TAG(s, R_NilValue);
-
-        value = R_possible_dispatch(call, op, args, rho, FALSE);
-        if(value) {
-            *ans = value;
-            return 1;
-        }
-        /* else go on to look for S3 methods */
+    if(useS4) {
+	/* Remove argument names to ensure positional matching */
+	if(isOps)
+	    for(s = args; s != R_NilValue; s = CDR(s)) SET_TAG(s, R_NilValue);
+	if(R_has_methods(op) && 
+	   (value = R_possible_dispatch(call, op, args, rho, FALSE))) {
+	       *ans = value;
+	       return 1;
+	}
+	/* else go on to look for S3 methods */
     }
 
     /* check whether we are processing the default method */
     if ( isSymbol(CAR(call)) ) {
-        if(strlen(CHAR(PRINTNAME(CAR(call)))) >= 512)
-           error(_("call name too long in '%s'"), CHAR(PRINTNAME(CAR(call))));
-        sprintf(lbuf, "%s", CHAR(PRINTNAME(CAR(call))) );
-        pt = strtok(lbuf, ".");
-        pt = strtok(NULL, ".");
+	if(strlen(CHAR(PRINTNAME(CAR(call)))) >= 512)
+	   error(_("call name too long in '%s'"), CHAR(PRINTNAME(CAR(call))));
+	sprintf(lbuf, "%s", CHAR(PRINTNAME(CAR(call))) );
+	pt = strtok(lbuf, ".");
+	pt = strtok(NULL, ".");
 
-        if( pt != NULL && !strcmp(pt, "default") )
-            return 0;
+	if( pt != NULL && !strcmp(pt, "default") )
+	    return 0;
     }
 
     if(isOps)
-        nargs = length(args);
+	nargs = length(args);
     else
-        nargs = 1;
+	nargs = 1;
 
     if( nargs == 1 && !isObject(CAR(args)) )
-        return 0;
+	return 0;
 
     if(!isObject(CAR(args)) && !isObject(CADR(args)))
-        return 0;
+	return 0;
 
     if(strlen(PRIMNAME(op)) >= 128)
-        error(_("generic name too long in '%s'"), PRIMNAME(op));
+	error(_("generic name too long in '%s'"), PRIMNAME(op));
     sprintf(generic, "%s", PRIMNAME(op) );
 
-    lclass = getAttrib(CAR(args), R_ClassSymbol);
+    lclass = IS_S4_OBJECT(CAR(args)) ? data_class_group(CAR(args))
+      : getAttrib(CAR(args), R_ClassSymbol);
 
     if( nargs == 2 )
-        rclass = getAttrib(CADR(args), R_ClassSymbol);
+	rclass = IS_S4_OBJECT(CADR(args)) ? data_class_group(CADR(args))
+      : getAttrib(CADR(args), R_ClassSymbol);
     else
-        rclass = R_NilValue;
+	rclass = R_NilValue;
 
     lsxp = R_NilValue; lgr = R_NilValue; lmeth = R_NilValue;
     rsxp = R_NilValue; rgr = R_NilValue; rmeth = R_NilValue;
 
     findmethod(lclass, group, generic, &lsxp, &lgr, &lmeth, &lwhich,
-               lbuf, rho);
+	       lbuf, rho);
     PROTECT(lgr);
+    if(isFunction(lsxp) && IS_S4_OBJECT(CAR(args))) {
+      if(lwhich > 0) {
+        value = CAR(args);
+	if(NAMED(value)) SET_NAMED(value, 2);
+	value = asS4(value, 0, 2);
+	/* This and the similar test below are not possible when people
+	   insist on writing S3 methods for S4 classes & so NOT getting
+	   S4 inheritance JMC 8.iii.09 */
+/* 	if(TYPEOF(value) == S4SXP) */
+/* 	  error(_("Non-vector S4 object as first argument to operator")); */
+	SETCAR(args, value);
+      }
+      else { /* Design error: S3 method written for S4 class */
+	/*TODO: should warn here? */
+      }
+    }
 
     if( nargs == 2 )
-        findmethod(rclass, group, generic, &rsxp, &rgr, &rmeth,
-                   &rwhich, rbuf, rho);
+	findmethod(rclass, group, generic, &rsxp, &rgr, &rmeth,
+		   &rwhich, rbuf, rho);
     else
-        rwhich = 0;
+	rwhich = 0;
+
+    if(isFunction(rsxp) && IS_S4_OBJECT(CADR(args))) {
+      if(rwhich > 0) {
+        value = CADR(args);
+	if(NAMED(value)) SET_NAMED(value, 2);
+	value = asS4(value, 0, 2);
+/* 	if(TYPEOF(value) == S4SXP) */
+/* 	  error(_("Non-vector S4 object as second argument to binary operator")); */
+	SETCADR(args, value);
+      }
+      else {
+	/*TODO: should warn here ? */
+      }
+    }
 
     PROTECT(rgr);
 
     if( !isFunction(lsxp) && !isFunction(rsxp) ) {
-        UNPROTECT(2);
-        return 0; /* no generic or group method so use default*/
+	UNPROTECT(2);
+	return 0; /* no generic or group method so use default*/
     }
 
     if( lsxp != rsxp ) {
-        if( isFunction(lsxp) && isFunction(rsxp) ) {
-            warning(_("Incompatible methods (\"%s\", \"%s\") for \"%s\""),
-                    CHAR(PRINTNAME(lmeth)), CHAR(PRINTNAME(rmeth)), generic);
-            UNPROTECT(2);
-            return 0;
-        }
-        /* if the right hand side is the one */
-        if( !isFunction(lsxp) ) { /* copy over the righthand stuff */
-            lsxp = rsxp;
-            lmeth = rmeth;
-            lgr = rgr;
-            lclass = rclass;
-            lwhich = rwhich;
-            strcpy(lbuf, rbuf);
-        }
+	if( isFunction(lsxp) && isFunction(rsxp) ) {
+	    warning(_("Incompatible methods (\"%s\", \"%s\") for \"%s\""),
+		    CHAR(PRINTNAME(lmeth)), CHAR(PRINTNAME(rmeth)), generic);
+	    UNPROTECT(2);
+	    return 0;
+	}
+	/* if the right hand side is the one */
+	if( !isFunction(lsxp) ) { /* copy over the righthand stuff */
+	    lsxp = rsxp;
+	    lmeth = rmeth;
+	    lgr = rgr;
+	    lclass = rclass;
+	    lwhich = rwhich;
+	    strcpy(lbuf, rbuf);
+	}
     }
 
     /* we either have a group method or a class method */
@@ -1421,21 +1493,21 @@ int DispatchGroup(const char* group, SEXP call, SEXP op, SEXP args, SEXP rho,
     PROTECT(m = allocVector(STRSXP,nargs));
     s = args;
     for (i = 0 ; i < nargs ; i++) {
-        t = getAttrib(CAR(s), R_ClassSymbol);
-        set = 0;
-        if (isString(t)) {
-            for (j = 0 ; j < length(t) ; j++) {
-                if (!strcmp(translateChar(STRING_ELT(t, j)),
-                            translateChar(STRING_ELT(lclass, lwhich)))) {
-                    SET_STRING_ELT(m, i, mkChar(lbuf));
-                    set = 1;
-                    break;
-                }
-            }
-        }
-        if( !set )
-            SET_STRING_ELT(m, i, R_BlankString);
-        s = CDR(s);
+	t = getAttrib(CAR(s), R_ClassSymbol);
+	set = 0;
+	if (isString(t)) {
+	    for (j = 0 ; j < length(t) ; j++) {
+		if (!strcmp(translateChar(STRING_ELT(t, j)),
+			    translateChar(STRING_ELT(lclass, lwhich)))) {
+		    SET_STRING_ELT(m, i, mkChar(lbuf));
+		    set = 1;
+		    break;
+		}
+	    }
+	}
+	if( !set )
+	    SET_STRING_ELT(m, i, R_BlankString);
+	s = CDR(s);
     }
 
     defineVar(install(".Method"), m, newrho);
@@ -1447,7 +1519,7 @@ int DispatchGroup(const char* group, SEXP call, SEXP op, SEXP args, SEXP rho,
     set = length(lclass) - lwhich;
     PROTECT(t = allocVector(STRSXP, set));
     for(j = 0 ; j < set ; j++ )
-        SET_STRING_ELT(t, j, duplicate(STRING_ELT(lclass, lwhich++)));
+	SET_STRING_ELT(t, j, duplicate(STRING_ELT(lclass, lwhich++)));
     defineVar(install(".Class"), t, newrho);
     UNPROTECT(1);
     defineVar(install(".GenericCallEnv"), rho, newrho);
@@ -1463,12 +1535,13 @@ int DispatchGroup(const char* group, SEXP call, SEXP op, SEXP args, SEXP rho,
     if (length(s) != length(args))
 	error(_("dispatch error in group dispatch"));
     for (m = s ; m != R_NilValue ; m = CDR(m), args = CDR(args) ) {
-        SET_PRVALUE(CAR(m), CAR(args));
-        /* ensure positional matching for operators */
-        if(isOps) SET_TAG(m, R_NilValue);
+	SET_PRVALUE(CAR(m), CAR(args));
+	/* ensure positional matching for operators */
+	if(isOps) SET_TAG(m, R_NilValue);
     }
 
     *ans = applyClosure(t, lsxp, s, rho, newrho);
     UNPROTECT(5);
     return 1;
 }
+
