@@ -629,6 +629,16 @@ SEXP attribute_hidden do_for(SEXP call, SEXP op, SEXP args, SEXP rho)
     defineVar(sym, RNIL, rho);
     prevJitState = jitEnterLoop(call, body);
 
+    /* deal with the case where we are iterating over a factor
+       we need to coerce to character - then iterate */
+
+    if( inherits(rhs, "factor") ) {
+        PROTECT(ans = asCharacterFactor(rhs));
+	rhs = ans;
+	UNPROTECT(2);  /* ans and rhs from above */
+        PROTECT(rhs);
+    }
+
     if (inJitBlock())
         PROTECT(ans = forjit(call, sym, rhs, body, rho));
     else
