@@ -117,7 +117,9 @@ extern SEXP framenames;
 	gc_inhibit_torture = 1 ; X ; gc_inhibit_torture = __t;}
 
 static void R_gc_internal(R_size_t size_needed);
+static void R_gc_full(R_size_t size_needed);
 static void mem_err_heap(R_size_t size);
+static void mem_err_malloc(R_size_t size);
 
 static SEXPREC UnmarkedNodeTemplate;
 #define NODE_IS_MARKED(s) (MARK(s)==1)
@@ -526,8 +528,8 @@ void ForwardNode(CSEXP s, SEXP *pforwarded_nodes) /* as above, but func */
 #define GET_FREE_NODE(s) CLASS_GET_FREE_NODE(0,s)
 
 /*-----------------------------------------------------------------------------
-   RA_TODO the following defs and related code will be removed.
-   TORTURE_FILL_FREE writes 0xee to free mem because no valgrind on WinXP.
+   RA_TODO the following defs and related code will eventually be removed.
+   TORTURE_FILL_FREE writes 0xee to free mem, needed because no valgrind on WinXP.
 */
 
 #define TORTURE_FILL_FREE 0
@@ -717,8 +719,12 @@ static void GetNewPage(int node_class)
     page_count = (R_PAGE_SIZE - sizeof(PAGE_HEADER)) / node_size;
 
     page = malloc(R_PAGE_SIZE);
-    if (page == NULL)
-	mem_err_heap((R_size_t) NodeClassSize[node_class]);
+    if (page == NULL) {
+	R_gc_full(0);
+	page = malloc(R_PAGE_SIZE);
+	if (page == NULL)
+	    mem_err_malloc((R_size_t) R_PAGE_SIZE);
+    }
 #ifdef R_MEMORY_PROFILING
     R_ReportNewPage();
 #endif
@@ -1629,6 +1635,11 @@ static void mem_err_cons(void)
     errorcall(R_NilValue, _("cons memory exhausted (limit reached?)"));
 }
 
+static void mem_err_malloc(R_size_t size)
+{
+    errorcall(R_NilValue, _("memory exhausted (limit reached?)"));
+}
+
 /* InitMemory : Initialise the memory to be used in R. */
 /* This includes: stack space, node space and vector space */
 
@@ -1981,7 +1992,7 @@ SEXP allocVector(SEXPTYPE type, R_len_t length)
 	actual_size=length;
 	break;
     case CHARSXP:
-	warning("use of allocVector(CHARSXP ...) is deprecated\n");
+	error("use of allocVector(CHARSXP ...) is defunct\n");
     case intCHARSXP:
 	size = BYTE2VEC(length + 1);
 	actual_size=length+1;
@@ -2096,7 +2107,7 @@ SEXP allocVector(SEXPTYPE type, R_len_t length)
 		    /* If we are near the address space limit, we
 		       might be short of address space.  So return
 		       all unused objects to malloc and try again. */
-		    R_gc_internal(alloc_size);
+		    R_gc_full(alloc_size);
 		    s = malloc(sizeof(SEXPREC_ALIGN) + size * sizeof(VECREC));
 		}
 		if (s != NULL) success = TRUE;
@@ -2210,6 +2221,12 @@ void R_gc(void)
     R_gc_internal(0);
 }
 
+static void R_gc_full(R_size_t size_needed)
+{
+    num_old_gens_to_collect = NUM_OLD_GENERATIONS;
+    R_gc_internal(size_needed);
+}
+
 #ifdef _R_HAVE_TIMING_
 double R_getClockIncrement(void);
 void R_getProcTime(double *data);
@@ -2597,13 +2614,6 @@ DL_FUNC R_ExternalPtrAddrFn(SEXP s)
 }
 
 
-#define USE_TYPE_CHECKING
-
-#if defined(USE_TYPE_CHECKING_STRICT) && !defined(USE_TYPE_CHECKING)
-# define USE_TYPE_CHECKING
-#endif
-
-
 /* The following functions are replacements for the accessor macros.
    They are used by code that does not have direct access to the
    internal representation of objects.  The assignment functions
@@ -2619,11 +2629,9 @@ int (TRACE)(SEXP x) { return TRACE(x); }
 int (LEVELS)(SEXP x) { return LEVELS(x); }
 
 void (SET_ATTRIB)(SEXP x, SEXP v) { 
-#ifdef USE_TYPE_CHECKING
     if(TYPEOF(v) != LISTSXP && TYPEOF(v) != NILSXP) 
 	error("value of 'SET_ATTRIB' must be a pairlist or NULL, not a '%s'", 
 	      type2char(TYPEOF(x)));
-#endif
     CHECK_OLD_TO_NEW(x, v); 
     ATTRIB(x) = v; 
 }
@@ -2650,89 +2658,62 @@ void (SETLENGTH)(SEXP x, int v) { SETLENGTH(x, v); }
 void (SET_TRUELENGTH)(SEXP x, int v) { SET_TRUELENGTH(x, v); }
 
 const char *(R_CHAR)(SEXP x) {
-#ifdef USE_TYPE_CHECKING
     if(TYPEOF(x) != CHARSXP) 
 	error("%s() can only be applied to a '%s', not a '%s'", 
 	      "CHAR", "CHARSXP", type2char(TYPEOF(x)));
-#endif
     return (const char *)CHAR(x);
 }
 
 SEXP (STRING_ELT)(SEXP x, int i) {
-#ifdef USE_TYPE_CHECKING
     if(TYPEOF(x) != STRSXP)
 	error("%s() can only be applied to a '%s', not a '%s'", 
 	      "STRING_ELT", "character vector", type2char(TYPEOF(x)));
-#endif
     return STRING_ELT(x, i);
 }
 
 SEXP (VECTOR_ELT)(SEXP x, int i) {
-#ifdef USE_TYPE_CHECKING_STRICT
     /* We need to allow vector-like types here */
     if(TYPEOF(x) != VECSXP && 
        TYPEOF(x) != EXPRSXP && 
        TYPEOF(x) != WEAKREFSXP)
 	error("%s() can only be applied to a '%s', not a '%s'", 
 	      "VECTOR_ELT", "list", type2char(TYPEOF(x)));
-#elif defined(USE_TYPE_CHECKING)
-    /* also allow STRSXP */
-    if(TYPEOF(x) != VECSXP && TYPEOF(x) != STRSXP &&
-       TYPEOF(x) != EXPRSXP && 
-       TYPEOF(x) != WEAKREFSXP)
-	error("%s() can only be applied to a '%s', not a '%s'", 
-	      "VECTOR_ELT", "list", type2char(TYPEOF(x)));
-#endif
     return VECTOR_ELT(x, i);
 }
 
 int *(LOGICAL)(SEXP x) {
-#ifdef USE_TYPE_CHECKING_STRICT
     if(TYPEOF(x) != LGLSXP)
 	error("%s() can only be applied to a '%s', not a '%s'",
 	      "LOGICAL",  "logical", type2char(TYPEOF(x)));
-#elif defined(USE_TYPE_CHECKING)
-    /* Currently harmless, and quite widely used */
-    if(TYPEOF(x) != LGLSXP && TYPEOF(x) != INTSXP)
-	error("%s() can only be applied to a '%s', not a '%s'",
-	      "LOGICAL",  "logical", type2char(TYPEOF(x)));
-#endif
   return LOGICAL(x); 
 }
 
+/* Maybe this should exclude logicals, but it is widely used */
 int *(INTEGER)(SEXP x) {
-#ifdef USE_TYPE_CHECKING
     if(TYPEOF(x) != INTSXP && TYPEOF(x) != LGLSXP)
         error("%s() can only be applied to a '%s', not a '%s'",
 	      "INTEGER", "integer", type2char(TYPEOF(x)));
-#endif
     return INTEGER(x); 
 }
 
 Rbyte *(RAW)(SEXP x) { 
-#ifdef USE_TYPE_CHECKING
     if(TYPEOF(x) != RAWSXP) 
 	error("%s() can only be applied to a '%s', not a '%s'", 
 	      "RAW", "raw", type2char(TYPEOF(x)));
-#endif
     return RAW(x); 
 }
 
 double *(REAL)(SEXP x) { 
-#ifdef USE_TYPE_CHECKING
     if(TYPEOF(x) != REALSXP) 
 	error("%s() can only be applied to a '%s', not a '%s'", 
 	      "REAL", "numeric", type2char(TYPEOF(x)));
-#endif
     return REAL(x); 
 }
 
 Rcomplex *(COMPLEX)(SEXP x) { 
-#ifdef USE_TYPE_CHECKING
     if(TYPEOF(x) != CPLXSXP)
 	error("%s() can only be applied to a '%s', not a '%s'", 
 	      "COMPLEX", "complex", type2char(TYPEOF(x)));
-#endif
     return COMPLEX(x); 
 }
 
@@ -2745,20 +2726,18 @@ SEXP *(VECTOR_PTR)(SEXP x)
 }
 
 void (SET_STRING_ELT)(SEXP x, int i, SEXP v) { 
-#ifdef USE_TYPE_CHECKING
     if(TYPEOF(x) != STRSXP)
 	error("%s() can only be applied to a '%s', not a '%s'", 
 	      "SET_STRING_ELT", "character vector", type2char(TYPEOF(x)));
+    /* NULL is used in subscript.c */
     if(TYPEOF(v) != CHARSXP && TYPEOF(v) != NILSXP)
        error("Value of SET_STRING_ELT() must be a 'CHARSXP' not a '%s'", 
 	     type2char(TYPEOF(v)));
-#endif
     CHECK_OLD_TO_NEW(x, v);
     STRING_ELT(x, i) = v; 
 }
 
 SEXP (SET_VECTOR_ELT)(SEXP x, int i, SEXP v) { 
-#ifdef USE_TYPE_CHECKING_STRICT
     /*  we need to allow vector-like types here */
     if(TYPEOF(x) != VECSXP &&
        TYPEOF(x) != EXPRSXP &&
@@ -2766,14 +2745,6 @@ SEXP (SET_VECTOR_ELT)(SEXP x, int i, SEXP v) {
 	error("%s() can only be applied to a '%s', not a '%s'", 
 	      "SET_VECTOR_ELT", "list", type2char(TYPEOF(x)));
     }
-#elif defined(USE_TYPE_CHECKING)
-    /* also allow STRSXP */
-    if(TYPEOF(x) != VECSXP && TYPEOF(x) != STRSXP &&
-       TYPEOF(x) != EXPRSXP && TYPEOF(x) != WEAKREFSXP) {
-	error("%s() can only be applied to a '%s', not a '%s'", 
-	      "SET_VECTOR_ELT", "list", type2char(TYPEOF(x)));
-    }
-#endif
     CHECK_OLD_TO_NEW(x, v); 
     return VECTOR_ELT(x, i) = v; 
 }
@@ -3140,10 +3111,8 @@ int Seql(SEXP a, SEXP b)
 {
     if (a == b) return 1;
     if (LENGTH(a) != LENGTH(b)) return 0;
-    if (IS_CACHED(a) && IS_CACHED(b) &&
-	(!ENC_KNOWN(a) || !ENC_KNOWN(b) || 
-	 ENC_KNOWN(a) == ENC_KNOWN(b)))
-	return 0;
+    if (IS_CACHED(a) && IS_CACHED(b) && ENC_KNOWN(a) == ENC_KNOWN(b))
+	 return 0;
     return !strcmp(translateCharUTF8(a), translateCharUTF8(b));
 }
 
@@ -3153,15 +3122,13 @@ int Seql(SEXP a, SEXP b)
 int Seql(SEXP a, SEXP b)
 {
     /* The only case where pointer comparisons do not suffice is where
-      we have two strings in different marked encodings, since in 
-      R > 2.8.0 ASCII strings in the cache are never marked.
-    */
+      we have two strings in different encodings (which must be
+      non-ASCII strings). Note that one of the strings could be marked
+      as unknown. */
     if (a == b) return 1;
     if (LENGTH(a) != LENGTH(b)) return 0;
     /* Leave this to compiler to optimize */
-    if (IS_CACHED(a) && IS_CACHED(b) &&
-	(!ENC_KNOWN(a) || !ENC_KNOWN(b) || 
-	 ENC_KNOWN(a) == ENC_KNOWN(b)))
+    if (IS_CACHED(a) && IS_CACHED(b) && ENC_KNOWN(a) == ENC_KNOWN(b))
 	return 0;
     return !strcmp(translateChar(a), translateChar(b));
 }
