@@ -57,8 +57,6 @@
  *  </FIXME>
  */
 
-/* <UTF8> char here is either ASCII or handled as a whole */
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -158,7 +156,7 @@ SEXP attribute_hidden do_prmatrix(SEXP call, SEXP op, SEXP args, SEXP rho)
 }/* do_prmatrix */
 
 /* .Internal(print.default(x, digits, quote, na.print, print.gap,
-                           right, max, useS4)) */
+			   right, max, useS4)) */
 SEXP attribute_hidden do_printdefault(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP x, naprint;
@@ -215,7 +213,7 @@ SEXP attribute_hidden do_printdefault(SEXP call, SEXP op, SEXP args, SEXP rho)
 
     R_print.useSource = asLogical(CAR(args));
     if(R_print.useSource == NA_LOGICAL)
-    	error(_("invalid '%s' argument"), "useSource");
+	error(_("invalid '%s' argument"), "useSource");
     if(R_print.useSource) R_print.useSource = USESOURCE;
     args = CDR(args);
 
@@ -227,7 +225,7 @@ SEXP attribute_hidden do_printdefault(SEXP call, SEXP op, SEXP args, SEXP rho)
 	callShow = TRUE;
 
     if(callShow) {
-	/* we need to get show from the methods namespace if it is 
+	/* we need to get show from the methods namespace if it is
 	   not visible on the search path. */
 	SEXP call, showS;
 	showS = findVar(install("show"), rho);
@@ -568,29 +566,10 @@ static void PrintExpression(SEXP s)
 	Rprintf("%s\n", CHAR(STRING_ELT(u, i))); /*translated */
 }
 
-
 /* PrintValueRec -- recursively print an SEXP
 
  * This is the "dispatching" function for  print.default()
  */
-
-static void PrintEnvir(SEXP rho)
-{
-    if (rho == R_GlobalEnv)
-	Rprintf("<environment: R_GlobalEnv>\n");
-    else if (rho == R_BaseEnv)
-    	Rprintf("<environment: base>\n");
-    else if (rho == R_EmptyEnv)
-    	Rprintf("<environment: R_EmptyEnv>\n");
-    else if (R_IsPackageEnv(rho))
-	Rprintf("<environment: %s>\n",
-		translateChar(STRING_ELT(R_PackageEnvName(rho), 0)));
-    else if (R_IsNamespaceEnv(rho))
-	Rprintf("<environment: namespace:%s>\n",
-		translateChar(STRING_ELT(R_NamespaceEnvSpec(rho), 0)));
-    else Rprintf("<environment: %p>\n", rho);
-}
-
 void attribute_hidden PrintValueRec(SEXP s, SEXP env)
 {
     int i;
@@ -606,7 +585,7 @@ void attribute_hidden PrintValueRec(SEXP s, SEXP env)
 	    if(TYPEOF(s) == S4SXP)
 		Rprintf("<S4 object without a class>\n");
 	    else
-		Rprintf("<Object of type '%s' with S4 bit but without a class>\n", 
+		Rprintf("<Object of type '%s' with S4 bit but without a class>\n",
 			type2char(TYPEOF(s)));
 	} else {
 	    SEXP pkg = getAttrib(s, install("package"));
@@ -679,17 +658,18 @@ void attribute_hidden PrintValueRec(SEXP s, SEXP env)
 	if (TYPEOF(s) == CLOSXP && isByteCode(BODY(s)))
 	    Rprintf("<bytecode: %p>\n", BODY(s));
 #endif
-	if (TYPEOF(s) == CLOSXP) t = CLOENV(s);
-	else t = R_GlobalEnv;
-	if (t != R_GlobalEnv)
-	    PrintEnvir(t);
+	if (TYPEOF(s) == CLOSXP) {
+	    t = CLOENV(s);
+	    if (t != R_GlobalEnv)
+		Rprintf("%s\n", EncodeEnvironment(t));
+	}
 	break;
     case ENVSXP:
-	PrintEnvir(s);
+	Rprintf("%s\n", EncodeEnvironment(s));
 	break;
     case PROMSXP:
-        Rprintf("<promise: %p>\n", s);
-        break;
+	Rprintf("<promise: %p>\n", s);
+	break;
     case DOTSXP:
 	Rprintf("<...>\n");
 	break;
@@ -714,7 +694,7 @@ void attribute_hidden PrintValueRec(SEXP s, SEXP env)
 		    const char *title = NULL;
 
 		    if (!isNull(nn))
-		        title = translateChar(STRING_ELT(nn, 0));
+			title = translateChar(STRING_ELT(nn, 0));
 
 		    printNamedVector(s, VECTOR_ELT(t, 0), R_print.quote, title);
 		}
@@ -832,7 +812,22 @@ static void printAttributes(SEXP s, SEXP env, Rboolean useSlots)
 		UNPROTECT(1);
 		goto nextattr;
 	    }
-	    if (isObject(CAR(a))) {
+	    if (isMethodsDispatchOn() && IS_S4_OBJECT(CAR(a))) {
+		SEXP s, showS;
+
+		showS = findVar(install("show"), env);
+		if(showS == R_UnboundValue) {
+		    SEXP methodsNS = R_FindNamespace(mkString("methods"));
+		    if(methodsNS == R_UnboundValue)
+			error("missing methods namespace: this should not happen");
+		    showS = findVarInFrame3(methodsNS, install("show"), TRUE);
+		    if(showS == R_UnboundValue)
+			error("missing show() in methods namespace: this should not happen");
+		}
+		PROTECT(s = lang2(showS, CAR(a)));
+		eval(s, env);
+		UNPROTECT(1);
+	    } else if (isObject(CAR(a))) {
 		/* Need to construct a call to
 		   print(CAR(a), digits)
 		   based on the R_print structure, then eval(call, env).
@@ -840,6 +835,8 @@ static void printAttributes(SEXP s, SEXP env, Rboolean useSlots)
 
 		   quote, right, gap should probably be included if
 		   they have non-missing values.
+
+		   This will not dispatch to show() as 'digits' is supplied.
 		*/
 		SEXP s, t, na_string = R_print.na_string,
 		    na_string_noquote = R_print.na_string_noquote;
@@ -885,13 +882,13 @@ void attribute_hidden PrintValueEnv(SEXP s, SEXP env)
     tagbuf[0] = '\0';
     PROTECT(s);
     if(isObject(s)) {
-	/* 
+	/*
 	   The intention here is to call show() on S4 objects, otherwise
 	   print(), so S4 methods for show() have precedence over those for
 	   print() to conform with the "green book", p. 332
 	*/
 	SEXP call, showS;
-        if(isMethodsDispatchOn() && IS_S4_OBJECT(s)) {
+	if(isMethodsDispatchOn() && IS_S4_OBJECT(s)) {
 	    /*
 	      Note that we cannot assume that show() is visible from
 	      'env', but we can assume there is a loaded "methods"
